"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const path = require("path");
const ctircLoader_1 = require("../options/ctircLoader");
const CTILogger_1 = require("../tools/CTILogger");
const CTIUtility_1 = require("../tools/CTIUtility");
const CommandModule_1 = require("./CommandModule");
class CleanCommandModule {
    do(cliCwd, passed) {
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = CTIUtility_1.isNotEmpty(passed.globOptions) && CTIUtility_1.isNotEmpty(passed.globOptions.cwd)
                ? passed.globOptions.cwd
                : process.cwd();
            const { readedFrom, option } = ctircLoader_1.ctircLoader({
                cwd: cliCwd,
                fromCliOption: passed,
                inputDir: cwd,
            });
            const logger = new CTILogger_1.CTILogger(option.verbose);
            logger.log('configuration from: ', readedFrom === '' ? 'default' : readedFrom);
            logger.log(chalk_1.default.yellowBright('Option: '), option);
            const indexFiles = yield CommandModule_1.CommandModule.promisify.glob('**/index.ts', {
                cwd,
                nonull: false,
            });
            const entrypointFiles = yield CommandModule_1.CommandModule.promisify.glob('**/entrypoint.ts', {
                cwd,
                nonull: false,
            });
            const concatted = indexFiles.concat(entrypointFiles);
            if (concatted.length === 0) {
                logger.flog(chalk_1.default.yellow(`Cannot find target file on working directory: ${cwd}`));
            }
            yield Promise.all(concatted.map((file) => {
                logger.log(chalk_1.default.redBright('delete file: '), path.join(cwd, file));
                return CommandModule_1.CommandModule.promisify.unlink(path.join(cwd, file));
            }));
            logger.flog(chalk_1.default.green(`clean succeeded: ${cwd}`));
        });
    }
    write(_param) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not Implemented');
        });
    }
}
exports.CleanCommandModule = CleanCommandModule;
//# sourceMappingURL=CleanCommandModule.js.map