"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const fs = require("fs");
const glob = require("glob");
const path = require("path");
const util = require("util");
class CommandModule {
    static targetFileFilter({ filenames, option, logger, }) {
        const targetExts = option.targetExts.map((ext) => (ext.startsWith('.') ? ext : `.${ext}`));
        try {
            const filteredFiles = filenames
                // Step 1, remove file by target extension
                .filter((filename) => targetExts.indexOf(path.extname(filename)) >= 0)
                // Step 2, remove exclude directory
                .filter((filename) => {
                return !option.excludes.reduce((result, exclude) => {
                    return result || path.dirname(filename).indexOf(exclude) >= 0;
                }, false);
            })
                // Step 3, remove declare file(*.d.ts)
                .filter((filename) => !filename.endsWith('.d.ts'))
                // Step 4, remove exclude pattern
                .filter((filename) => {
                return !option.fileExcludePatterns.reduce((result, excludePattern) => {
                    return result || filename.indexOf(excludePattern) >= 0;
                }, false);
            })
                // Step 5, remove index file(index.ts, index.tsx etc ...)
                .filter((filename) => {
                return !option.targetExts
                    .map((ext) => `index.${ext}`)
                    .reduce((result, indexFile) => {
                    return result || filename.indexOf(indexFile) >= 0;
                }, false);
            })
                // Step 6, remove current directory
                .filter((filename) => {
                return filename !== '.';
            });
            return filteredFiles;
        }
        catch (err) {
            logger.error(chalk.default.redBright('Error occured: ', err));
            return [];
        }
    }
    static getDefaultOption(cwd) {
        return {
            addNewline: true,
            excludes: ['@types', 'typings', '__test__', '__tests__', 'node_modules'],
            fileExcludePatterns: [],
            fileFirst: false,
            globOptions: {
                cwd: cwd || process.cwd(),
                dot: true,
                nonull: true,
            },
            includeCWD: true,
            quote: "'",
            targetExts: ['ts', 'tsx'],
            useSemicolon: true,
            useTimestamp: false,
            verbose: false,
        };
    }
}
CommandModule.promisify = {
    glob: util.promisify(glob),
    readDir: util.promisify(fs.readdir),
    stat: util.promisify(fs.stat),
    unlink: util.promisify(fs.unlink),
    writeFile: util.promisify(fs.writeFile),
};
exports.CommandModule = CommandModule;
//# sourceMappingURL=CommandModule.js.map