"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const debug_1 = require("debug");
const moment = require("moment");
const path = require("path");
const ctircLoader_1 = require("../options/ctircLoader");
const CTILogger_1 = require("../tools/CTILogger");
const CTIUtility_1 = require("../tools/CTIUtility");
const exportStatement_1 = require("../tools/exportStatement");
const CommandModule_1 = require("./CommandModule");
const log = debug_1.default('cti:CreateCommandModule');
class CreateCommandModule {
    do(cliCwd, passed) {
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = CTIUtility_1.isNotEmpty(passed.globOptions) && CTIUtility_1.isNotEmpty(passed.globOptions.cwd)
                ? passed.globOptions.cwd
                : process.cwd();
            const { readedFrom, option } = ctircLoader_1.ctircLoader({
                cwd: cliCwd,
                fromCliOption: passed,
                inputDir: cwd,
            });
            const logger = new CTILogger_1.CTILogger(option.verbose);
            logger.log(chalk_1.default.yellowBright('Configuration from: '), readedFrom === '' ? 'default' : readedFrom);
            try {
                logger.log(chalk_1.default.yellowBright('Option: '), option);
                const targetFileGlob = option.targetExts.map((ext) => `*.${ext}`).join('|');
                const allTsFiles = yield CommandModule_1.CommandModule.promisify.glob(`**/+(${targetFileGlob})`, option.globOptions);
                const tsFiles = CommandModule_1.CommandModule.targetFileFilter({
                    logger,
                    option,
                    filenames: allTsFiles,
                });
                const dupLibDirs = tsFiles
                    .filter((tsFile) => tsFile.split('/').length > 1)
                    .map((tsFile) => {
                    const splitted = tsFile.split('/');
                    const allPath = Array(splitted.length - 1)
                        .fill(0)
                        .map((_, index) => index + 1)
                        .map((index) => {
                        const a = splitted.slice(0, index).join('/');
                        return a;
                    });
                    return allPath;
                })
                    .reduce((aggregated, libPath) => {
                    return aggregated.concat(libPath);
                }, []);
                const dirSet = new Set();
                dupLibDirs.forEach((dir) => dirSet.add(dir));
                tsFiles.map((tsFile) => path.dirname(tsFile)).forEach((dir) => dirSet.add(dir));
                const tsDirs = Array.from(dirSet);
                if (option.includeCWD &&
                    tsDirs.findIndex((dir) => path.resolve(dir) === path.resolve('.')) < 0) {
                    tsDirs.push('.');
                }
                tsDirs.sort((left, right) => {
                    const llen = left.split('/').length;
                    const rlen = right.split('/').length;
                    if (llen > rlen) {
                        return -1;
                    }
                    if (llen < rlen) {
                        return 1;
                    }
                    return 0;
                });
                log('tsDirs:: ', tsDirs);
                yield Promise.all(tsDirs.map((tsDir) => this.write({ option, logger, directory: tsDir, directories: tsDirs })));
                logger.flog(chalk_1.default.green(`create succeeded: ${option.globOptions.cwd}`));
            }
            catch (err) {
                log(err.message);
                log(err.stack);
                logger.ferror(chalk_1.default.red(err.message));
            }
        });
    }
    write({ directory, option, logger, }) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                logger.log(chalk_1.default.yellowBright('Current working directory: ', directory));
                const resolvePath = path.resolve(option.globOptions.cwd || __dirname);
                const elements = yield CommandModule_1.CommandModule.promisify.readDir(path.join(resolvePath, directory));
                const stats = yield Promise.all(elements.map((target) => CommandModule_1.CommandModule.promisify.stat(path.join(resolvePath, directory, target))));
                const statMap = elements.reduce((map, element, index) => {
                    map[element] = stats[index];
                    return map;
                }, {});
                const targets = elements
                    .filter((element) => statMap[element].isDirectory() || element !== 'index.ts')
                    .filter((element) => statMap[element].isDirectory() || element !== 'entrypoint.ts');
                const categorized = targets.reduce((result, target) => {
                    if (statMap[target].isDirectory()) {
                        result.dir.push(target);
                    }
                    else {
                        result.allFiles.push(target);
                    }
                    return result;
                }, { dir: [], allFiles: [] });
                categorized.dir.sort();
                categorized.allFiles = CommandModule_1.CommandModule.targetFileFilter({
                    logger,
                    option,
                    filenames: categorized.allFiles,
                });
                const files = categorized.allFiles.filter((element) => {
                    return !option.fileExcludePatterns.reduce((result, excludePattern) => {
                        return result || element.indexOf(excludePattern) >= 0;
                    }, false);
                });
                files.sort();
                const sorted = (() => {
                    if (option.fileFirst) {
                        return categorized.allFiles.concat(categorized.dir);
                    }
                    return categorized.dir.concat(files);
                })();
                const getExport = exportStatement_1.getExportStatementCreator(option, logger);
                const exportString = sorted.map((target) => getExport(target));
                const comment = (() => {
                    if (option.useTimestamp) {
                        return `// created from ${option.quote}create-ts-index${option.quote} ${moment(new Date()).format('YYYY-MM-DD HH:mm')}\n\n`;
                    }
                    return `// created from ${option.quote}create-ts-index${option.quote}\n\n`;
                })();
                const fileContent = comment + CTIUtility_1.addNewline(option, exportString.join('\n'));
                logger.log(chalk_1.default.green('created: '), `${path.join(resolvePath, directory, 'index.ts')}`);
                yield CommandModule_1.CommandModule.promisify.writeFile(path.join(resolvePath, directory, 'index.ts'), fileContent, 'utf8');
            }
            catch (err) {
                log(err.message);
                log(err.stack);
                logger.error(chalk_1.default.red('indexWriter: ', err.message));
            }
        });
    }
}
exports.CreateCommandModule = CreateCommandModule;
//# sourceMappingURL=CreateCommandModule.js.map