"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const debug_1 = require("debug");
const fs = require("fs");
const json5 = require("json5");
const moment = require("moment");
const path = require("path");
const CreateTsIndexOption_1 = require("../options/CreateTsIndexOption");
const ctircLoader_1 = require("../options/ctircLoader");
const CTILogger_1 = require("../tools/CTILogger");
const CTIUtility_1 = require("../tools/CTIUtility");
const log = debug_1.default('cti:InitCommandModule');
class InitCommandModule {
    do(cliCwd, passed) {
        return __awaiter(this, void 0, void 0, function* () {
            const cwd = CTIUtility_1.isNotEmpty(passed.globOptions) && CTIUtility_1.isNotEmpty(passed.globOptions.cwd)
                ? passed.globOptions.cwd
                : process.cwd();
            const { readedFrom, option } = ctircLoader_1.ctircLoader({
                cwd: cliCwd,
                fromCliOption: passed,
                inputDir: null,
            });
            log('Option: ', option.useTimestamp);
            const logger = new CTILogger_1.CTILogger(option.verbose);
            logger.log('configuration from: ', readedFrom === '' ? 'default' : readedFrom);
            try {
                const defaultOption = CreateTsIndexOption_1.getDefailtICreateTsIndexOption(cwd);
                const stringified = json5.stringify(defaultOption, null, 2);
                const headContent = (() => {
                    if (option.useTimestamp) {
                        return `// created from ${option.quote}create-ts-index${option.quote} ${moment(new Date()).format('YYYY-MM-DD HH:mm')}`;
                    }
                    return `// created from ${option.quote}create-ts-index${option.quote}`;
                })();
                const addNewline = (() => {
                    if (option.addNewline) {
                        return '\n';
                    }
                    return '';
                })();
                yield new Promise((resolve, reject) => {
                    fs.writeFile(path.join(cwd, '.ctirc'), `${headContent}\n\n${stringified}${addNewline}`, (err) => {
                        if (CTIUtility_1.isNotEmpty(err)) {
                            return reject(err);
                        }
                        logger.flog(chalk_1.default.green(`.ctirc create succeeded: ${option.globOptions.cwd}`));
                        return resolve();
                    });
                });
            }
            catch (err) {
                logger.error(chalk_1.default.red('indexWriter: ', err.message));
                logger.error(chalk_1.default.red('indexWriter: ', err.stack));
            }
        });
    }
    write(_args) {
        return __awaiter(this, void 0, void 0, function* () {
            throw new Error('Not implements');
        });
    }
}
exports.InitCommandModule = InitCommandModule;
//# sourceMappingURL=InitCommandModule.js.map