#!/usr/bin/env node
"use strict";
// tslint:disable no-console no-string-literal
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const chalk = require("chalk");
const debug_1 = require("debug");
const fs = require("fs");
const path = require("path");
const yargs_1 = require("yargs");
const CleanCommandModule_1 = require("./commands/CleanCommandModule");
const CreateCommandModule_1 = require("./commands/CreateCommandModule");
const EntrypointCommandModule_1 = require("./commands/EntrypointCommandModule");
const InitCommandModule_1 = require("./commands/InitCommandModule");
const EN_CLIOPTION_1 = require("./EN_CLIOPTION");
const CreateTsIndexOption_1 = require("./options/CreateTsIndexOption");
const CTIUtility_1 = require("./tools/CTIUtility");
const log = debug_1.default('cti:cti-cli');
const version = (() => {
    if (fs.existsSync(path.join(__dirname, 'package.json'))) {
        const packageJSON = JSON.parse(fs.readFileSync(path.join(__dirname, 'package.json')).toString());
        return packageJSON.version;
    }
    if (fs.existsSync(path.join('.', 'package.json'))) {
        const packageJSON = JSON.parse(fs.readFileSync(path.join('.', 'package.json')).toString());
        return packageJSON.version;
    }
    return '1.7.2';
})();
const optionMap = {
    [EN_CLIOPTION_1.EN_CLI_OPTION.FILEFIRST]: {
        alias: 'f',
        describe: 'export list create filefirst, no option false, option true',
        type: 'boolean',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE]: {
        alias: 'n',
        describe: 'deside add newline file ending. no option true, option false',
        type: 'boolean',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON]: {
        alias: 's',
        describe: 'deside use semicolon line ending. no option true, option false',
        type: 'boolean',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD]: {
        alias: 'c',
        describe: 'deside include cwd directory in task. no option true, option false',
        type: 'boolean',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP]: {
        alias: 't',
        describe: `deside use timestamp(YYYY-MM-DD HH:mm) top line comment. \nno option false, option true`,
        type: 'boolean',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES]: {
        alias: 'e',
        array: true,
        describe: `pass exclude directory. default exclude directory is ['@types', 'typings', '__test__', '__tests__']`,
        type: 'string',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES]: {
        alias: 'i',
        array: true,
        describe: 'pass exclude pattern of filename. default exclude directory is "[]"',
        type: 'string',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS]: {
        alias: 'x',
        array: true,
        describe: `pass include extname. default extname is ["ts", "tsx"]. extname \npass without dot charactor.`,
        type: 'string',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE]: {
        alias: 'v',
        describe: 'verbose logging message. to option false, option true',
        type: 'boolean',
    },
    [EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE]: {
        alias: 'q',
        describe: "deside quote character. default quote character is '",
        type: 'string',
    },
};
const parser = yargs_1.default({
    addnewline: true,
    cwds: [],
    excludes: ['@types', 'typings', '__test__', '__tests__', 'node_modules'],
    fileexcludes: [],
    filefirst: false,
    includecwd: true,
    quote: "'",
    targetexts: ['ts', 'tsx'],
    usesemicolon: true,
    usetimestamp: false,
    verbose: false,
})
    .command('$0 [cwds...]', 'create index.ts file in working directory', (args) => {
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.FILEFIRST, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.FILEFIRST]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE]);
    return args;
}, (args) => __awaiter(this, void 0, void 0, function* () {
    const cwds = args.cwds;
    const cliCwd = process.cwd();
    if (!cwds) {
        console.log(chalk.default.magenta('Enter working directory, '));
        console.log(chalk.default.red('cti [working directory]'));
        process.exit(1);
    }
    if (typeof cwds === 'string') {
        const createCommand = new CreateCommandModule_1.CreateCommandModule();
        const options = CreateTsIndexOption_1.createOptionBuilder(args, cwds);
        return createCommand.do(cliCwd, options);
    }
    if (typeof cwds !== 'string' && Array.isArray(cwds)) {
        return Promise.all(cwds
            .filter((cwd) => fs.existsSync(cwd))
            .map((cwd) => {
            const createCommand = new CreateCommandModule_1.CreateCommandModule();
            const options = CreateTsIndexOption_1.createOptionBuilder(args, cwd);
            return createCommand.do(cliCwd, options);
        }));
    }
}))
    .command('create [cwds...]', 'create index.ts file in working directory', (args) => {
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.FILEFIRST, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.FILEFIRST]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE]);
    return args;
}, (args) => __awaiter(this, void 0, void 0, function* () {
    const cwds = args['cwds'];
    const cliCwd = process.cwd();
    log('cli option: ', args);
    if (!cwds) {
        console.log(chalk.default.magenta('Enter working directory, '));
        console.log(chalk.default.red('cti [working directory]'));
        process.exit(1);
    }
    if (typeof cwds === 'string') {
        const createCommand = new CreateCommandModule_1.CreateCommandModule();
        const options = CreateTsIndexOption_1.createOptionBuilder(args, cwds);
        return createCommand.do(cliCwd, options);
    }
    if (typeof cwds !== 'string' && Array.isArray(cwds)) {
        return Promise.all(cwds
            .filter((cwd) => fs.existsSync(cwd))
            .map((cwd) => {
            const createCommand = new CreateCommandModule_1.CreateCommandModule();
            const options = CreateTsIndexOption_1.createOptionBuilder(args, cwd);
            return createCommand.do(cliCwd, options);
        }));
    }
}))
    .command('entrypoint [cwds...]', 'create entrypoint.ts file in working directory', (args) => {
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_SEMICOLON]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.INCLUDE_CWD]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.EXCLUDES]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.FILE_EXCLUDES]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.TARGET_EXTS]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.QUOTE]);
    return args;
}, (args) => __awaiter(this, void 0, void 0, function* () {
    const cwds = args.cwds;
    const cliCwd = process.cwd();
    if (!cwds) {
        console.log(chalk.default.magenta('Enter working directory, '));
        console.log(chalk.default.red('cti [working directory]'));
        process.exit(1);
    }
    if (typeof cwds === 'string') {
        const entrypointCommand = new EntrypointCommandModule_1.EntrypointCommandModule();
        const options = CreateTsIndexOption_1.entrypointOptionBuilder(args, cwds);
        return entrypointCommand.do(cliCwd, options);
    }
    if (typeof cwds !== 'string' && Array.isArray(cwds)) {
        return Promise.all(cwds
            .filter((cwd) => fs.existsSync(cwd))
            .map((cwd) => {
            const entrypointCommand = new EntrypointCommandModule_1.EntrypointCommandModule();
            const options = CreateTsIndexOption_1.entrypointOptionBuilder(args, cwd);
            return entrypointCommand.do(cliCwd, options);
        }));
    }
}))
    .command('init [cwds...]', 'create .ctirc file in working directory', (args) => {
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.ADD_NEWLINE]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.USE_TIMESTAMP]);
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE]);
    return args;
}, (args) => __awaiter(this, void 0, void 0, function* () {
    const cwds = CTIUtility_1.isNotEmpty(args.cwds) ? args.cwds : [process.cwd()];
    const cliCwd = process.cwd();
    log('init command start, ', cwds);
    if (typeof cwds === 'string') {
        const initCommandModule = new InitCommandModule_1.InitCommandModule();
        const options = CreateTsIndexOption_1.initOptionBuilder(args, cwds);
        return initCommandModule.do(cliCwd, options);
    }
    if (typeof cwds !== 'string' && Array.isArray(cwds)) {
        return Promise.all(cwds
            .filter((cwd) => fs.existsSync(cwd))
            .map((cwd) => {
            const initCommandModule = new InitCommandModule_1.InitCommandModule();
            const options = CreateTsIndexOption_1.initOptionBuilder(args, cwd);
            return initCommandModule.do(cliCwd, options);
        }));
    }
}))
    .command('clean [cwds...]', 'clean index.ts or entrypoint.ts file in working directory', (args) => {
    args.option(EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE, optionMap[EN_CLIOPTION_1.EN_CLI_OPTION.VERBOSE]);
    return args;
}, (args) => __awaiter(this, void 0, void 0, function* () {
    const cwds = args['cwds'];
    const cliCwd = process.cwd();
    if (!cwds) {
        console.log(chalk.default.magenta('Enter working directory, '));
        console.log(chalk.default.red('cti [working directory]'));
        process.exit(1);
    }
    if (typeof cwds === 'string') {
        const cleanCommand = new CleanCommandModule_1.CleanCommandModule();
        const options = CreateTsIndexOption_1.cleanOptionBuilder(args, cwds);
        yield cleanCommand.do(cliCwd, options);
    }
    if (typeof cwds !== 'string' && Array.isArray(cwds)) {
        return Promise.all(cwds
            .filter((cwd) => fs.existsSync(cwd))
            .map((cwd) => {
            const cleanCommand = new CleanCommandModule_1.CleanCommandModule();
            const options = CreateTsIndexOption_1.cleanOptionBuilder(args, cwd);
            return cleanCommand.do(cliCwd, options);
        }));
    }
}))
    .version(version, 'version', 'display version information')
    .help();
// tslint:disable-next-line
parser.argv;
//# sourceMappingURL=cti.js.map