"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("debug");
const CTIUtility_1 = require("../tools/CTIUtility");
const log = debug_1.default('cti:CreateTsIndexOption');
function createOptionBuilder(args, cwd) {
    log('create option building');
    return {
        addNewline: args.addnewline,
        excludes: args.excludes,
        fileExcludePatterns: args.fileexcludes,
        fileFirst: args.filefirst,
        globOptions: {
            cwd,
        },
        includeCWD: args.includecwd,
        quote: args.quote,
        targetExts: args.targetexts,
        useSemicolon: args.usesemicolon,
        useTimestamp: args.usetimestamp,
        verbose: args.verbose,
    };
}
exports.createOptionBuilder = createOptionBuilder;
function entrypointOptionBuilder(args, cwd) {
    log('entrypoint option building');
    return {
        addNewline: args.addnewline,
        excludes: args.excludes,
        fileExcludePatterns: args.fileexcludes,
        fileFirst: false,
        globOptions: {
            cwd,
        },
        includeCWD: args.includecwd,
        quote: args.quote,
        targetExts: args.targetexts,
        useSemicolon: args.usesemicolon,
        useTimestamp: args.usetimestamp,
        verbose: args.verbose,
    };
}
exports.entrypointOptionBuilder = entrypointOptionBuilder;
function cleanOptionBuilder(args, cwd) {
    log('clean option building');
    return {
        addNewline: false,
        excludes: [],
        fileExcludePatterns: [],
        fileFirst: false,
        globOptions: {
            cwd,
        },
        includeCWD: false,
        quote: "'",
        targetExts: [],
        useSemicolon: false,
        useTimestamp: false,
        verbose: args.verbose,
    };
}
exports.cleanOptionBuilder = cleanOptionBuilder;
function initOptionBuilder(args, cwd) {
    log('init option building');
    return {
        addNewline: args.addnewline,
        excludes: [],
        fileExcludePatterns: [],
        fileFirst: false,
        globOptions: {
            cwd,
        },
        includeCWD: false,
        quote: "'",
        targetExts: [],
        useSemicolon: false,
        useTimestamp: args.usetimestamp,
        verbose: args.verbose,
    };
}
exports.initOptionBuilder = initOptionBuilder;
function getDefailtICreateTsIndexOption(cwd) {
    return {
        addNewline: true,
        excludes: ['@types', 'typings', '__test__', '__tests__', 'node_modules'],
        fileExcludePatterns: [],
        fileFirst: false,
        globOptions: {
            cwd: CTIUtility_1.isNotEmpty(cwd) ? cwd : process.cwd(),
            dot: true,
            nonull: true,
        },
        includeCWD: true,
        quote: "'",
        targetExts: ['ts', 'tsx'],
        useSemicolon: true,
        useTimestamp: false,
        verbose: false,
    };
}
exports.getDefailtICreateTsIndexOption = getDefailtICreateTsIndexOption;
class CreateTsIndexOption {
    static factory({ option, partialOption, }) {
        if (CTIUtility_1.isNotEmpty(option)) {
            const createTsIndexOption = new CreateTsIndexOption(option);
            return createTsIndexOption;
        }
        if (CTIUtility_1.isNotEmpty(partialOption)) {
            const createTsIndexOption = CreateTsIndexOption.getOption(partialOption);
            return createTsIndexOption;
        }
        const defaultOption = getDefailtICreateTsIndexOption();
        return new CreateTsIndexOption(defaultOption);
    }
    static getOption(passed) {
        const option = getDefailtICreateTsIndexOption();
        option.fileFirst = CTIUtility_1.isNotEmpty(passed.fileFirst) ? passed.fileFirst : option.fileFirst;
        option.addNewline = CTIUtility_1.isNotEmpty(passed.addNewline) ? passed.addNewline : option.addNewline;
        option.useSemicolon = CTIUtility_1.isNotEmpty(passed.useSemicolon)
            ? passed.useSemicolon
            : option.useSemicolon;
        option.useTimestamp = CTIUtility_1.isNotEmpty(passed.useTimestamp)
            ? passed.useTimestamp
            : option.useTimestamp;
        option.includeCWD = CTIUtility_1.isNotEmpty(passed.includeCWD) ? passed.includeCWD : option.includeCWD;
        option.fileExcludePatterns = CTIUtility_1.isNotEmpty(passed.fileExcludePatterns)
            ? passed.fileExcludePatterns
            : option.fileExcludePatterns;
        option.excludes = CTIUtility_1.isNotEmpty(passed.excludes) ? passed.excludes : option.excludes;
        option.targetExts = CTIUtility_1.isNotEmpty(passed.targetExts) ? passed.targetExts : option.targetExts;
        // remove sorting
        // option.targetExts = option.targetExts.sort((l, r) => r.length - l.length);
        if (CTIUtility_1.isNotEmpty(passed.globOptions)) {
            option.globOptions.cwd = CTIUtility_1.isNotEmpty(passed.globOptions.cwd)
                ? passed.globOptions.cwd
                : option.globOptions.cwd;
            option.globOptions.nonull = CTIUtility_1.isNotEmpty(passed.globOptions.nonull)
                ? passed.globOptions.nonull
                : option.globOptions.nonull;
            option.globOptions.dot = CTIUtility_1.isNotEmpty(passed.globOptions.dot)
                ? passed.globOptions.dot
                : option.globOptions.dot;
        }
        option.verbose = CTIUtility_1.isNotEmpty(passed.verbose) ? passed.verbose : option.verbose;
        return new CreateTsIndexOption(option);
    }
    constructor(fileFirst, addNewline, useSemicolon, useTimestamp, includeCWD, excludes, fileExcludePatterns, targetExts, globOptions, quote, verbose) {
        if (typeof fileFirst === 'boolean') {
            this.fileFirst = fileFirst;
            this.addNewline = CTIUtility_1.isNotEmpty(addNewline) ? addNewline : true;
            this.useSemicolon = CTIUtility_1.isNotEmpty(useSemicolon) ? useSemicolon : true;
            this.useTimestamp = CTIUtility_1.isNotEmpty(useTimestamp) ? useTimestamp : false;
            this.includeCWD = CTIUtility_1.isNotEmpty(includeCWD) ? includeCWD : true;
            this.excludes = CTIUtility_1.isNotEmpty(excludes)
                ? excludes
                : ['@types', 'typings', '__test__', '__tests__', 'node_modules'];
            this.fileExcludePatterns = CTIUtility_1.isNotEmpty(fileExcludePatterns) ? fileExcludePatterns : [];
            this.targetExts = CTIUtility_1.isNotEmpty(targetExts) ? targetExts : ['ts', 'tsx'];
            this.globOptions = CTIUtility_1.isNotEmpty(globOptions)
                ? globOptions
                : {
                    cwd: process.cwd(),
                    dot: true,
                    nonull: true,
                };
            this.quote = CTIUtility_1.isNotEmpty(quote) ? quote : "'";
            this.verbose = CTIUtility_1.isNotEmpty(verbose) ? verbose : false;
        }
        else {
            const { fileFirst: _fileFirst, addNewline: _addNewline, useSemicolon: _useSemicolon, useTimestamp: _useTimestamp, includeCWD: _includeCWD, excludes: _excludes, fileExcludePatterns: _fileExcludePatterns, targetExts: _targetExts, globOptions: _globOptions, quote: _quote, verbose: _verbose, } = fileFirst || getDefailtICreateTsIndexOption();
            this.fileFirst = _fileFirst;
            this.addNewline = _addNewline;
            this.useSemicolon = _useSemicolon;
            this.useTimestamp = _useTimestamp;
            this.includeCWD = _includeCWD;
            this.excludes = _excludes;
            this.fileExcludePatterns = _fileExcludePatterns;
            this.targetExts = _targetExts;
            this.globOptions = _globOptions;
            this.quote = _quote;
            this.verbose = _verbose;
        }
    }
}
exports.CreateTsIndexOption = CreateTsIndexOption;
//# sourceMappingURL=CreateTsIndexOption.js.map