"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const debug_1 = require("debug");
const fs = require("fs");
const json5 = require("json5");
const merge = require("merge");
const path = require("path");
const CTIUtility_1 = require("../tools/CTIUtility");
const CreateTsIndexOption_1 = require("./CreateTsIndexOption");
const log = debug_1.default('cti:ctircLoader');
const CTIRC_FILENAME = '.ctirc';
function ctircLoader({ inputDir, cwd, fromCliOption, }) {
    log(`inputDir: (${inputDir}) /cwd: (${cwd})`);
    // Step01. Read configuration file on parametered input directory
    const targetDirs = (() => {
        const target = new Set();
        if (CTIUtility_1.isNotEmpty(inputDir) &&
            fs.existsSync(inputDir) &&
            fs.existsSync(path.join(inputDir, CTIRC_FILENAME))) {
            target.add(inputDir);
        }
        if (fs.existsSync(cwd) && fs.existsSync(path.join(cwd, CTIRC_FILENAME))) {
            target.add(cwd);
        }
        return Array.from(target);
    })();
    log('targetDirs: ', targetDirs);
    // Stop02. Load configuration from .ctirc
    try {
        if (targetDirs.length <= 0) {
            return {
                option: new CreateTsIndexOption_1.CreateTsIndexOption(CreateTsIndexOption_1.CreateTsIndexOption.getOption(fromCliOption)),
                readedFrom: 'from cli option',
            };
        }
        const fromConfigfiles = targetDirs.map((targetDir) => {
            const ctircPath = path.join(targetDir, CTIRC_FILENAME);
            const rawConf = fs.readFileSync(ctircPath).toString();
            const fromConfigfile = Object.assign({}, json5.parse(rawConf), { globOptions: { cwd: inputDir } });
            return fromConfigfile;
        });
        const mergedCTIRC = fromConfigfiles.reduce((src, dst) => merge.recursive(true, src, dst));
        mergedCTIRC.__for_debug_from = 'from-config-file';
        if (Object.keys(fromCliOption).length > 0) {
            fromCliOption.__for_debug_from = 'from-cli-option';
        }
        const option = [
            Object.assign({ __for_debug_from: 'from-default-config' }, CreateTsIndexOption_1.getDefailtICreateTsIndexOption()),
            mergedCTIRC,
            fromCliOption,
        ].reduce((src, dst) => merge.recursive(true, src, dst));
        log('final option-1: ', mergedCTIRC);
        log('final option-2: ', fromCliOption);
        log('final option-3: ', option);
        return { option: CreateTsIndexOption_1.CreateTsIndexOption.factory({ option }), readedFrom: 'from config file' };
    }
    catch (err) {
        log(err.message);
        log(err.stack);
        const option = new CreateTsIndexOption_1.CreateTsIndexOption();
        return { option, readedFrom: '' };
    }
}
exports.ctircLoader = ctircLoader;
//# sourceMappingURL=ctircLoader.js.map